
VERSION: str = "0.15.0"
REPOSITORY_URL: str = f"https://pypi.org/pypi/mindgard/json"
AUTH0_DOMAIN= "login.sandbox.mindgard.ai"
AUTH0_CLIENT_ID= "U0OT7yZLJ4GEyabar11BENeQduu4MaNO"
AUTH0_AUDIENCE="https://marketplace-orchestrator.com"
ALGORITHMS = ['RS256']

